## Brian Blaylock
## October 1, 2021

"""
I have no idea what I'm doing yet. First time making a test.
"""
import numpy as np


def test_sqrt():
    num = 25
    assert np.sqrt(num) == 5


def test_str():
    a = "Herbie"
    b = "Herbie"
    assert a == b
